@@ -5,6 +5,7 @@ import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.InputStream;
 import java.net.URLClassLoader;
+import java.nio.file.Files;
 import java.nio.file.Path;
 import java.nio.file.Paths;
 import java.util.ArrayList;
@@ -133,10 +134,10 @@ public class TestNG {
   private ITestRunnerFactory m_testRunnerFactory;
 
   // These listeners can be overridden from the command line
-  private List<IClassListener> m_classListeners = Lists.newArrayList();
-  private List<ITestListener> m_testListeners = Lists.newArrayList();
-  private List<ISuiteListener> m_suiteListeners = Lists.newArrayList();
-  private Set<IReporter> m_reporters = Sets.newHashSet();
+  private final Map<Class<? extends IClassListener>, IClassListener> m_classListeners = Maps.newHashMap();
+  private final Map<Class<? extends ITestListener>, ITestListener> m_testListeners = Maps.newHashMap();
+  private final Map<Class<? extends ISuiteListener>, ISuiteListener> m_suiteListeners = Maps.newHashMap();
+  private final Map<Class<? extends IReporter>, IReporter> m_reporters = Maps.newHashMap();
 
   protected static final int HAS_FAILURE = 1;
   protected static final int HAS_SKIPPED = 2;
@@ -161,7 +162,8 @@ public class TestNG {
 
   private ITestObjectFactory m_objectFactory;
 
-  private List<IInvokedMethodListener> m_invokedMethodListeners = Lists.newArrayList();
+  private final Map<Class<? extends IInvokedMethodListener>, IInvokedMethodListener> m_invokedMethodListeners = Maps
+      .newHashMap();
 
   private Integer m_dataProviderThreadCount = null;
 
@@ -177,9 +179,8 @@ public class TestNG {
   protected long m_end;
   protected long m_start;
 
-  private List<IExecutionListener> m_executionListeners = Lists.newArrayList();
-
-  private List<IAlterSuiteListener> m_alterSuiteListeners= Lists.newArrayList();
+  private final Map<Class<? extends IExecutionListener>, IExecutionListener> m_executionListeners = Maps.newHashMap();
+  private final Map<Class<? extends IAlterSuiteListener>, IAlterSuiteListener> m_alterSuiteListeners= Maps.newHashMap();
 
   private boolean m_isInitialized = false;
 
@@ -270,9 +271,16 @@ public class TestNG {
     	//to parse the suite files (<suite-file>), if any
     	for (XmlSuite s: m_suites) {
         for (String suiteFile : s.getSuiteFiles()) {
-            Path rootPath = Paths.get(s.getFileName()).getParent();
             try {
-                Collection<XmlSuite> childSuites = getParser(rootPath.resolve(suiteFile).normalize().toString()).parse();
+                Collection<XmlSuite> childSuites;
+                if (s.getFileName() != null) {
+                  Path rootPath = Paths.get(s.getFileName()).getParent();
+                  try (InputStream is = Files.newInputStream(rootPath.resolve(suiteFile))) {
+                    childSuites = getParser(is).parse();
+                  }
+                } else {
+                  childSuites = getParser(suiteFile).parse();
+                }
                 for (XmlSuite cSuite : childSuites){
                     cSuite.setParentSuite(s);
                     s.getChildSuites().add(cSuite);
@@ -425,7 +433,7 @@ public class TestNG {
     }
 
     if (tests.size() == 0) {
-      return s;
+      throw new TestNGException("The test(s) <" + testNames.toString() + "> cannot be found.");
     }
     else {
       XmlSuite result = (XmlSuite) s.clone();
@@ -683,8 +691,8 @@ public class TestNG {
    * @param classes A list of classes, which must be either ISuiteListener,
    * ITestListener or IReporter
    */
-  public void setListenerClasses(List<Class> classes) {
-    for (Class cls: classes) {
+  public void setListenerClasses(List<Class<? extends ITestNGListener>> classes) {
+    for (Class<? extends ITestNGListener> cls: classes) {
       addListener(ClassHelper.newInstance(cls));
     }
   }
@@ -692,7 +700,7 @@ public class TestNG {
   /**
    * @deprecated Use addListener(ITestNGListener) instead
    */
-  // TODO remove later
+  // TODO remove later /!\ Caution: IntelliJ is using it. Check with @akozlova before removing it
   @Deprecated
   public void addListener(Object listener) {
     if (! (listener instanceof ITestNGListener))
@@ -704,21 +712,33 @@ public class TestNG {
     addListener((ITestNGListener) listener);
   }
 
+  private static <E> void maybeAddListener(Map<Class<? extends E>, E> map, Class<? extends E> type, E value) {
+    if (map.containsKey(value.getClass())) {
+      LOGGER.warn("Ignoring duplicate listener : " + value.getClass().getName());
+    } else {
+      map.put(type, value);
+    }
+  }
+
   public void addListener(ITestNGListener listener) {
     if (listener == null) {
       return;
     }
     if (listener instanceof ISuiteListener) {
-      m_suiteListeners.add((ISuiteListener) listener);
+      ISuiteListener suite = (ISuiteListener) listener;
+      maybeAddListener(m_suiteListeners, suite.getClass(),  suite);
     }
     if (listener instanceof ITestListener) {
-      m_testListeners.add((ITestListener) listener);
+      ITestListener test = (ITestListener) listener;
+      maybeAddListener(m_testListeners, test.getClass(), test);
     }
     if (listener instanceof IClassListener) {
-      m_classListeners.add((IClassListener) listener);
+      IClassListener clazz = (IClassListener) listener;
+      maybeAddListener(m_classListeners, clazz.getClass(), clazz);
     }
     if (listener instanceof IReporter) {
-      m_reporters.add((IReporter) listener);
+      IReporter reporter = (IReporter) listener;
+      maybeAddListener(m_reporters, reporter.getClass(), reporter);
     }
     if (listener instanceof IAnnotationTransformer) {
       setAnnotationTransformer((IAnnotationTransformer) listener);
@@ -727,7 +747,8 @@ public class TestNG {
       m_methodInterceptors.add((IMethodInterceptor) listener);
     }
     if (listener instanceof IInvokedMethodListener) {
-      m_invokedMethodListeners.add((IInvokedMethodListener) listener);
+      IInvokedMethodListener method = (IInvokedMethodListener) listener;
+      maybeAddListener(m_invokedMethodListeners, method.getClass(), method);
     }
     if (listener instanceof IHookable) {
       setHookable((IHookable) listener);
@@ -736,13 +757,15 @@ public class TestNG {
       setConfigurable((IConfigurable) listener);
     }
     if (listener instanceof IExecutionListener) {
-      m_executionListeners.add((IExecutionListener) listener);
+      IExecutionListener execution = (IExecutionListener) listener;
+      maybeAddListener(m_executionListeners, execution.getClass(), execution);
     }
     if (listener instanceof IConfigurationListener) {
       getConfiguration().addConfigurationListener((IConfigurationListener) listener);
     }
     if (listener instanceof IAlterSuiteListener) {
-      m_alterSuiteListeners.add((IAlterSuiteListener) listener);
+      IAlterSuiteListener alter = (IAlterSuiteListener) listener;
+      maybeAddListener(m_alterSuiteListeners, alter.getClass(), alter);
     }
   }
 
@@ -801,15 +824,17 @@ public class TestNG {
   }
 
   public Set<IReporter> getReporters() {
-    return m_reporters;
+    //This will now cause a different behavior for consumers of this method because unlike before they are no longer
+    //going to be getting the original set but only a copy of it (since we internally moved from Sets to Maps)
+    return Sets.newHashSet(m_reporters.values());
   }
 
   public List<ITestListener> getTestListeners() {
-    return m_testListeners;
+    return Lists.newArrayList(m_testListeners.values());
   }
 
   public List<ISuiteListener> getSuiteListeners() {
-    return m_suiteListeners;
+    return Lists.newArrayList(m_suiteListeners.values());
   }
 
   /** If m_verbose gets set, it will override the verbose setting in testng.xml */
@@ -829,7 +854,7 @@ public class TestNG {
 
   private boolean m_randomizeSuites = Boolean.FALSE;
 
-  private boolean m_preserveOrder = false;
+  private Boolean m_preserveOrder = XmlSuite.DEFAULT_PRESERVE_ORDER;
   private Boolean m_groupByInstances;
 
   private IConfiguration m_configuration;
@@ -857,7 +882,7 @@ public class TestNG {
 
       for (XmlSuite s : m_cmdlineSuites) {
         for (XmlTest t : s.getTests()) {
-          t.setPreserveOrder(String.valueOf(m_preserveOrder));
+          t.setPreserveOrder(m_preserveOrder);
         }
         m_suites.add(s);
         if (m_groupByInstances != null) {
@@ -894,24 +919,32 @@ public class TestNG {
     List<XmlSuite> suites = m_cmdlineSuites != null ? m_cmdlineSuites : m_suites;
     if (hasIncludedGroups || hasExcludedGroups) {
       for (XmlSuite s : suites) {
-        //set on each test, instead of just the first one of the suite
-        for (XmlTest t : s.getTests()) {
-          if(hasIncludedGroups) {
-            t.setIncludedGroups(Arrays.asList(m_includedGroups));
-          }
-          if(hasExcludedGroups) {
-            t.setExcludedGroups(Arrays.asList(m_excludedGroups));
-          }
-        }
+        initializeCommandLineSuitesGroups(s, hasIncludedGroups, m_includedGroups, hasExcludedGroups, m_excludedGroups);
       }
     }
   }
+
+  private static void initializeCommandLineSuitesGroups(XmlSuite s,
+                                                        boolean hasIncludedGroups, String[] m_includedGroups,
+                                                        boolean hasExcludedGroups, String[] m_excludedGroups) {
+    if (hasIncludedGroups) {
+      s.setIncludedGroups(Arrays.asList(m_includedGroups));
+    }
+    if (hasExcludedGroups) {
+      s.setExcludedGroups(Arrays.asList(m_excludedGroups));
+    }
+    for (XmlSuite child : s.getChildSuites()) {
+      initializeCommandLineSuitesGroups(child, hasIncludedGroups, m_includedGroups, hasExcludedGroups, m_excludedGroups);
+    }
+  }
   private void addReporter(Class<? extends IReporter> r) {
-    m_reporters.add(ClassHelper.newInstance(r));
+    if (!m_reporters.containsKey(r)) {
+      m_reporters.put(r, ClassHelper.newInstance(r));
+    }
   }
 
   private void initializeDefaultListeners() {
-    m_testListeners.add(new ExitCodeListener(this));
+    m_testListeners.put(ExitCodeListener.class, new ExitCodeListener(this));
 
     if (m_useDefaultListeners) {
       addReporter(SuiteHTMLReporter.class);
@@ -942,18 +975,7 @@ public class TestNG {
     // Install the listeners found in the suites
     //
     for (XmlSuite s : m_suites) {
-      for (String listenerName : s.getListeners()) {
-        Class<?> listenerClass = ClassHelper.forName(listenerName);
-
-        // If specified listener does not exist, a TestNGException will be thrown
-        if(listenerClass == null) {
-          throw new TestNGException("Listener " + listenerName
-              + " was not found in project's classpath");
-        }
-
-        Object listener = ClassHelper.newInstance(listenerClass);
-        addListener(listener);
-      }
+      addListeners(s);
 
       //
       // Install the method selectors
@@ -980,6 +1002,26 @@ public class TestNG {
     m_configuration.setObjectFactory(factory);
   }
 
+  private void addListeners(XmlSuite s) {
+    for (String listenerName : s.getListeners()) {
+      Class<?> listenerClass = ClassHelper.forName(listenerName);
+
+      // If specified listener does not exist, a TestNGException will be thrown
+      if(listenerClass == null) {
+        throw new TestNGException("Listener " + listenerName + " was not found in project's classpath");
+      }
+
+      ITestNGListener listener = (ITestNGListener) ClassHelper.newInstance(listenerClass);
+      addListener(listener);
+    }
+
+    // Add the child suite listeners
+    List<XmlSuite> childSuites = s.getChildSuites();
+    for (XmlSuite c : childSuites) {
+      addListeners(c);
+    }
+  }
+
   /**
    * Using reflection to remain Java 5 compliant.
    */
@@ -1067,21 +1109,21 @@ public class TestNG {
 
     sanityCheck();
 
-    List<ISuite> suiteRunners = null;
-
-    runSuiteAlterationListeners();
     runExecutionListeners(true /* start */);
 
+    runSuiteAlterationListeners();
+
     m_start = System.currentTimeMillis();
-    suiteRunners = runSuites();
+    List<ISuite> suiteRunners = runSuites();
 
     m_end = System.currentTimeMillis();
-    runExecutionListeners(false /* finish */);
 
     if(null != suiteRunners) {
       generateReports(suiteRunners);
     }
 
+    runExecutionListeners(false /* finish */);
+
     if(!m_hasTests) {
       setStatus(HAS_NO_TEST);
       if (TestRunner.getVerbose() > 1) {
@@ -1097,7 +1139,7 @@ public class TestNG {
    * This method can be overridden by subclass. <br/>
    * For example, DistributedTestNG to run in master/slave mode according to commandline args. 
    * </p>
-   * @return
+   * @return - List of suites that were run as {@link ISuite} objects.
    * @since 6.9.11 when moving distributed/remote classes out into separate project
    */
   protected List<ISuite> runSuites() {
@@ -1105,8 +1147,8 @@ public class TestNG {
   }
 
   private void runSuiteAlterationListeners() {
-    for (List<IAlterSuiteListener> listeners
-        : Arrays.asList(m_alterSuiteListeners, m_configuration.getAlterSuiteListeners())) {
+    for (Collection<IAlterSuiteListener> listeners
+        : Arrays.asList(m_alterSuiteListeners.values(), m_configuration.getAlterSuiteListeners())) {
       for (IAlterSuiteListener l : listeners) {
         l.alter(m_suites);
       }
@@ -1114,8 +1156,8 @@ public class TestNG {
   }
 
   private void runExecutionListeners(boolean start) {
-    for (List<IExecutionListener> listeners
-        : Arrays.asList(m_executionListeners, m_configuration.getExecutionListeners())) {
+    for (Collection<IExecutionListener> listeners
+        : Arrays.asList(m_executionListeners.values(), m_configuration.getExecutionListeners())) {
       for (IExecutionListener l : listeners) {
         if (start) l.onExecutionStart();
         else l.onExecutionFinish();
@@ -1149,7 +1191,7 @@ public class TestNG {
   }
 
   private void generateReports(List<ISuite> suiteRunners) {
-    for (IReporter reporter : m_reporters) {
+    for (IReporter reporter : m_reporters.values()) {
       try {
         long start = System.currentTimeMillis();
         reporter.generateReport(m_suites, suiteRunners, m_outputDir);
@@ -1201,7 +1243,7 @@ public class TestNG {
         IThreadWorkerFactory<ISuite> factory = new SuiteWorkerFactory(suiteRunnerMap,
           0 /* verbose hasn't been set yet */, getDefaultSuiteName());
         GraphThreadPoolExecutor<ISuite> pooledExecutor =
-                new GraphThreadPoolExecutor<>(suiteGraph, factory, m_suiteThreadPoolSize,
+                new GraphThreadPoolExecutor<>("suites", suiteGraph, factory, m_suiteThreadPoolSize,
                         m_suiteThreadPoolSize, Integer.MAX_VALUE, TimeUnit.MILLISECONDS,
                         new LinkedBlockingQueue<Runnable>());
 
@@ -1281,7 +1323,7 @@ public class TestNG {
     }
     else {
       for (XmlSuite childSuite : xmlSuite.getChildSuites()) {
-        suiteGraph.addEdge(parentSuiteRunner, suiteRunnerMap.get(childSuite));
+        suiteGraph.addEdge(0, parentSuiteRunner, suiteRunnerMap.get(childSuite));
         populateSuiteGraph(suiteGraph, suiteRunnerMap, childSuite);
       }
     }
@@ -1341,11 +1383,11 @@ public class TestNG {
         m_testRunnerFactory,
         m_useDefaultListeners,
         m_methodInterceptors,
-        m_invokedMethodListeners,
-        m_testListeners,
-        m_classListeners);
+        m_invokedMethodListeners.values(),
+        m_testListeners.values(),
+        m_classListeners.values());
 
-    for (ISuiteListener isl : m_suiteListeners) {
+    for (ISuiteListener isl : m_suiteListeners.values()) {
       result.addListener(isl);
     }
 
@@ -1485,10 +1527,13 @@ public class TestNG {
         sep = ",";
       }
       String[] strs = Utils.split(cla.listener, sep);
-      List<Class> classes = Lists.newArrayList();
+      List<Class<? extends ITestNGListener>> classes = Lists.newArrayList();
 
       for (String cls : strs) {
-        classes.add(ClassHelper.fileToClass(cls));
+        Class<?> clazz = ClassHelper.fileToClass(cls);
+        if (ITestNGListener.class.isAssignableFrom(clazz)) {
+          classes.add((Class<? extends ITestNGListener>) clazz);
+        }
       }
 
       setListenerClasses(classes);
@@ -1683,7 +1728,7 @@ public class TestNG {
     if (instance != null) {
       addListener(instance);
     } else {
-      LOGGER.warn("Could not find reporte class : " + reporterConfig.getClassName());
+      LOGGER.warn("Could not find reporter class : " + reporterConfig.getClassName());
     }
   }
 
@@ -1837,7 +1882,7 @@ public class TestNG {
   /**
    * Sets the policy for whether or not to ever invoke a configuration method again after
    * it has failed once. Possible values are defined in {@link XmlSuite}.  The default
-   * value is {@link XmlSuite.FailurePolicy#SKIP}.
+   * value is {@link org.testng.xml.XmlSuite.FailurePolicy#SKIP}
    * @param failurePolicy the configuration failure policy
    */
   public void setConfigFailurePolicy(XmlSuite.FailurePolicy failurePolicy) {
@@ -2027,7 +2072,7 @@ public class TestNG {
   //
 
   private URLClassLoader m_serviceLoaderClassLoader;
-  private List<ITestNGListener> m_serviceLoaderListeners = Lists.newArrayList();
+  private Map<Class<? extends ITestNGListener>, ITestNGListener> serviceLoaderListeners = Maps.newHashMap();
 
   /*
    * Used to test ServiceClassLoader
@@ -2040,14 +2085,16 @@ public class TestNG {
    * Used to test ServiceClassLoader
    */
   private void addServiceLoaderListener(ITestNGListener l) {
-    m_serviceLoaderListeners.add(l);
+    if (! serviceLoaderListeners.containsKey(l.getClass())) {
+      serviceLoaderListeners.put(l.getClass(), l);
+    }
   }
 
   /*
    * Used to test ServiceClassLoader
    */
   public List<ITestNGListener> getServiceLoaderListeners() {
-    return m_serviceLoaderListeners;
+    return Lists.newArrayList(serviceLoaderListeners.values());
   }
 
   //
