@@ -7,7 +7,6 @@ import java.util.Collection;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.Date;
-import java.util.EnumSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -45,7 +44,6 @@ import org.testng.internal.Utils;
 import org.testng.internal.XmlMethodSelector;
 import org.testng.internal.annotations.AnnotationHelper;
 import org.testng.internal.annotations.IAnnotationFinder;
-import org.testng.internal.annotations.IListeners;
 import org.testng.internal.thread.graph.GraphThreadPoolExecutor;
 import org.testng.internal.thread.graph.IThreadWorkerFactory;
 import org.testng.internal.thread.graph.IWorker;
@@ -87,7 +85,7 @@ public class TestRunner
   transient private boolean m_skipFailedInvocationCounts;
 
   transient private Collection<IInvokedMethodListener> m_invokedMethodListeners = Lists.newArrayList();
-  transient private List<IClassListener> m_classListeners = Lists.newArrayList();
+  transient private final Map<Class<? extends IClassListener>, IClassListener> m_classListeners = Maps.newHashMap();
 
   /**
    * All the test methods we found, associated with their respective classes.
@@ -149,6 +147,10 @@ public class TestRunner
   private transient IConfiguration m_configuration;
   private IMethodInterceptor builtinInterceptor;
 
+  private enum PriorityWeight {
+    groupByInstance, preserveOrder, priority, dependsOnGroups, dependsOnMethods
+  }
+
   protected TestRunner(IConfiguration configuration,
                     ISuite suite,
                     XmlTest test,
@@ -190,8 +192,8 @@ public class TestRunner
     setVerbose(test.getVerbose());
 
 
-    boolean preserveOrder = "true".equalsIgnoreCase(test.getPreserveOrder());
-    m_methodInterceptors = new ArrayList<IMethodInterceptor>();
+    boolean preserveOrder = test.getPreserveOrder();
+    m_methodInterceptors = new ArrayList<>();
     builtinInterceptor = preserveOrder ? new PreserveOrderMethodInterceptor() : new InstanceOrderingMethodInterceptor();
 
     m_packageNamesFromXml= test.getXmlPackages();
@@ -203,12 +205,15 @@ public class TestRunner
 
     m_annotationFinder= annotationFinder;
     m_invokedMethodListeners = invokedMethodListeners;
-    m_classListeners = classListeners;
+    m_classListeners.clear();
+    for (IClassListener classListener : classListeners) {
+      m_classListeners.put(classListener.getClass(), classListener);
+    }
     m_invoker = new Invoker(m_configuration, this, this, m_suite.getSuiteState(),
         m_skipFailedInvocationCounts, invokedMethodListeners, classListeners);
 
-    if (suite.getParallel() != null) {
-      log(3, "Running the tests in '" + test.getName() + "' with parallel mode:" + suite.getParallel());
+    if (test.getParallel() != null) {
+      log(3, "Running the tests in '" + test.getName() + "' with parallel mode:" + test.getParallel());
     }
 
     setOutputDirectory(outputDirectory);
@@ -249,6 +254,9 @@ public class TestRunner
 
     initListeners();
     addConfigurationListener(m_confListener);
+    for (IConfigurationListener cl : m_configuration.getConfigurationListeners()) {
+      addConfigurationListener(cl);
+    }
   }
 
   private static class ListenerHolder {
@@ -331,6 +339,9 @@ public class TestRunner
 
     // Instantiate all the listeners
     for (Class<? extends ITestNGListener> c : listenerClasses) {
+      if (IClassListener.class.isAssignableFrom(c) && m_classListeners.containsKey(c)) {
+          continue;
+      }
       ITestNGListener listener = listenerFactory != null ? listenerFactory.createListener(c) : null;
       if (listener == null) {
         listener = ClassHelper.newInstance(c);
@@ -390,7 +401,6 @@ public class TestRunner
 
     ClassInfoMap classMap = new ClassInfoMap(m_testClassesFromXml);
     m_testClassFinder= new TestNGClassFinder(classMap,
-                                             null,
                                              m_xmlTest,
                                              m_configuration,
                                              this);
@@ -714,7 +724,7 @@ public class TestRunner
       if (parallel) {
         if (graph.getNodeCount() > 0) {
           GraphThreadPoolExecutor<ITestNGMethod> executor =
-                  new GraphThreadPoolExecutor<>(graph, this,
+                  new GraphThreadPoolExecutor<>("test=" + xmlTest.getName(), graph, this,
                           threadCount, threadCount, 0, TimeUnit.MILLISECONDS,
                           new LinkedBlockingQueue<Runnable>());
           executor.run();
@@ -771,12 +781,12 @@ public class TestRunner
     for (IMethodInstance imi : methodInstances) {
       result.add(imi.getMethod());
     }
-    
-    //Since an interceptor is involved, we would need to ensure that the ClassMethodMap object is in sync with the 
+
+    //Since an interceptor is involved, we would need to ensure that the ClassMethodMap object is in sync with the
     //output of the interceptor, else @AfterClass doesn't get executed at all when interceptors are involved.
     //so let's update the current classMethodMap object with the list of methods obtained from the interceptor.
     this.m_classMethodMap = new ClassMethodMap(result, null);
-    
+
     return result.toArray(new ITestNGMethod[result.size()]);
   }
 
@@ -854,8 +864,6 @@ public class TestRunner
       }
     }
 
-    // Sort by priorities
-    Collections.sort(result);
     return result;
   }
 
@@ -882,7 +890,7 @@ public class TestRunner
           m_groupMethods,
           m_classMethodMap,
           this,
-          m_classListeners);
+          new ArrayList<>(m_classListeners.values()));
       result.add(tmw);
     }
 
@@ -921,7 +929,7 @@ public class TestRunner
         m_groupMethods,
         m_classMethodMap,
         this,
-        m_classListeners);
+        new ArrayList<>(m_classListeners.values()));
   }
 
   private IMethodInstance[] findClasses(List<IMethodInstance> methodInstances, Class<?> c) {
@@ -996,12 +1004,23 @@ public class TestRunner
 
   private DynamicGraph<ITestNGMethod> createDynamicGraph(ITestNGMethod[] methods) {
     DynamicGraph<ITestNGMethod> result = new DynamicGraph<>();
-    result.setComparator(new Comparator<ITestNGMethod>() {
-      @Override
-      public int compare(ITestNGMethod o1, ITestNGMethod o2) {
-        return o1.getPriority() - o2.getPriority();
+
+    ListMultiMap<Integer, ITestNGMethod> methodsByPriority = Maps.newListMultiMap();
+    for (ITestNGMethod method : methods) {
+      methodsByPriority.put(method.getPriority(), method);
+    }
+    List<Integer> availablePriorities = Lists.newArrayList(methodsByPriority.keySet());
+    Collections.sort(availablePriorities);
+    Integer previousPriority = methods.length > 0 ? availablePriorities.get(0) : 0;
+    for (int i = 1; i < availablePriorities.size(); i++) {
+      Integer currentPriority = availablePriorities.get(i);
+      for (ITestNGMethod p0Method : methodsByPriority.get(previousPriority)) {
+        for (ITestNGMethod p1Method : methodsByPriority.get(currentPriority)) {
+          result.addEdge(PriorityWeight.priority.ordinal(), p1Method, p0Method);
+        }
       }
-    });
+      previousPriority = currentPriority;
+    }
 
     DependencyMap dependencyMap = new DependencyMap(methods);
 
@@ -1019,7 +1038,7 @@ public class TestRunner
           for (String d : dependentMethods) {
             ITestNGMethod dm = dependencyMap.getMethodDependingOn(d, m);
             if (m != dm){
-            	result.addEdge(m, dm);
+            	result.addEdge(PriorityWeight.dependsOnMethods.ordinal(), m, dm);
             }
           }
         }
@@ -1036,7 +1055,7 @@ public class TestRunner
                 + "\" depends on nonexistent group \"" + d + "\"");
           }
           for (ITestNGMethod ddm : dg) {
-            result.addEdge(m, ddm);
+            result.addEdge(PriorityWeight.dependsOnGroups.ordinal(), m, ddm);
           }
         }
       }
@@ -1049,7 +1068,7 @@ public class TestRunner
     // sequentially.
     if (! hasDependencies
         && getCurrentXmlTest().getParallel() == XmlSuite.ParallelMode.NONE
-        && "true".equalsIgnoreCase(getCurrentXmlTest().getPreserveOrder())) {
+        && getCurrentXmlTest().getPreserveOrder()) {
       // If preserve-order was specified and the class order is A, B
       // create a new set of dependencies where each method of B depends
       // on all the methods of A
@@ -1058,31 +1077,24 @@ public class TestRunner
 
       for (Map.Entry<ITestNGMethod, List<ITestNGMethod>> es : classDependencies.entrySet()) {
         for (ITestNGMethod dm : es.getValue()) {
-          result.addEdge(dm, es.getKey());
+          result.addEdge(PriorityWeight.preserveOrder.ordinal(), dm, es.getKey());
         }
       }
     }
 
     // Group by instances
     if (getCurrentXmlTest().getGroupByInstances()) {
-      ListMultiMap<ITestNGMethod, ITestNGMethod> instanceDependencies
-          = createInstanceDependencies(methods, getCurrentXmlTest());
-
+      ListMultiMap<ITestNGMethod, ITestNGMethod> instanceDependencies = createInstanceDependencies(methods);
       for (Map.Entry<ITestNGMethod, List<ITestNGMethod>> es : instanceDependencies.entrySet()) {
-        for (ITestNGMethod dm : es.getValue()) {
-          result.addEdge(dm, es.getKey());
-        }
+        result.addEdge(PriorityWeight.groupByInstance.ordinal(), es.getKey(), es.getValue());
       }
-
     }
 
     return result;
   }
 
-  private ListMultiMap<ITestNGMethod, ITestNGMethod> createInstanceDependencies(
-      ITestNGMethod[] methods, XmlTest currentXmlTest)
-  {
-    ListMultiMap<Object, ITestNGMethod> instanceMap = Maps.newListMultiMap();
+  private ListMultiMap<ITestNGMethod, ITestNGMethod> createInstanceDependencies(ITestNGMethod[] methods) {
+    ListMultiMap<Object, ITestNGMethod> instanceMap = Maps.newSortedListMultiMap();
     for (ITestNGMethod m : methods) {
       instanceMap.put(m.getInstance(), m);
     }
@@ -1158,10 +1170,8 @@ public class TestRunner
         // index
         String classDependedUpon = indexedClasses2.get(index - 1);
         List<ITestNGMethod> methodsDependedUpon = methodsFromClass.get(classDependedUpon);
-        if (methodsDependedUpon != null) {
-          for (ITestNGMethod mdu : methodsDependedUpon) {
-            result.put(mdu, m);
-          }
+        for (ITestNGMethod mdu : methodsDependedUpon) {
+          result.put(mdu, m);
         }
       }
     }
@@ -1193,11 +1203,15 @@ public class TestRunner
    */
   private void fireEvent(boolean isStart) {
     for (ITestListener itl : m_testListeners) {
-      if (isStart) {
-        itl.onStart(this);
-      }
-      else {
-        itl.onFinish(this);
+      try {
+        if (isStart) {
+          itl.onStart(this);
+        }
+        else {
+          itl.onFinish(this);
+        }
+      } catch (Exception e) {
+        e.printStackTrace();
       }
     }
   }
@@ -1450,25 +1464,19 @@ public class TestRunner
     if (listener instanceof IMethodInterceptor) {
       m_methodInterceptors.add((IMethodInterceptor) listener);
     }
-    if (listener instanceof ISuiteListener) {
-      m_suite.addListener(listener);
-    }
-    if (listener instanceof IInvokedMethodListener) {
-      m_suite.addListener(listener);
-    }
     if (listener instanceof ITestListener) {
       // At this point, the field m_testListeners has already been used in the creation
       addTestListener((ITestListener) listener);
     }
     if (listener instanceof IClassListener) {
-      m_classListeners.add((IClassListener) listener);
+      IClassListener classListener = (IClassListener) listener;
+      if (!m_classListeners.containsKey(classListener.getClass())) {
+        m_classListeners.put(classListener.getClass(), classListener);
+      }
     }
     if (listener instanceof IConfigurationListener) {
       addConfigurationListener((IConfigurationListener) listener);
     }
-    if (listener instanceof IReporter) {
-      m_suite.addListener(listener);
-    }
     if (listener instanceof IConfigurable) {
       m_configuration.setConfigurable((IConfigurable) listener);
     }
@@ -1480,6 +1488,7 @@ public class TestRunner
       iel.onExecutionStart();
       m_configuration.addExecutionListener(iel);
     }
+    m_suite.addListener(listener);
   }
 
   /**
@@ -1595,8 +1604,7 @@ public class TestRunner
 
   @Override
   public List<Module> getGuiceModules(Class<? extends Module> cls) {
-    List<Module> result = m_guiceModules.get(cls);
-    return result;
+    return m_guiceModules.get(cls);
   }
 
   private void addGuiceModule(Class<? extends Module> cls, Module module) {
