@@ -6,10 +6,7 @@ import java.lang.annotation.Target;
 /**
  * Marks a method as a factory that returns objects that will be used by TestNG
  * as Test classes.  The method must return Object[].
- *
- * @author <a href="mailto:cedric&#64;beust.com">Cedric Beust</a>
  */
-
 @Retention(java.lang.annotation.RetentionPolicy.RUNTIME)
 @Target({java.lang.annotation.ElementType.METHOD, java.lang.annotation.ElementType.CONSTRUCTOR})
 public @interface Factory {
@@ -20,13 +17,13 @@ public @interface Factory {
    * @deprecated Use @Parameters
    */
   @Deprecated
-  public String[] parameters() default {};
+  String[] parameters() default {};
 
   /**
    * The name of the data provider for this test method.
    * @see org.testng.annotations.DataProvider
    */
-  public String dataProvider() default "";
+  String dataProvider() default "";
 
   /**
    * The class where to look for the data provider.  If not
@@ -35,10 +32,12 @@ public @interface Factory {
    * If this attribute is specified, the data provider method
    * needs to be static on the specified class.
    */
-  public Class<?> dataProviderClass() default Object.class;
+  Class<?> dataProviderClass() default Object.class;
 
   /**
    * Whether this factory is enabled.
    */
-  public boolean enabled() default true;
+  boolean enabled() default true;
+
+  int[] indices() default {};
 }
