@@ -139,22 +139,6 @@ public abstract class BaseTestMethod implements ITestNGMethod {
     m_testClass = tc;
   }
 
-  @Override
-  public int compareTo(Object o) {
-    int result = -2;
-    Class<?> thisClass = getRealClass();
-    Class<?> otherClass = ((ITestNGMethod) o).getRealClass();
-    if (this == o || equals(o)) {
-      result = 0;
-    } else if (thisClass.isAssignableFrom(otherClass)) {
-      result = -1;
-    } else if (otherClass.isAssignableFrom(thisClass)) {
-      result = 1;
-    }
-
-    return result;
-  }
-
   /**
    * {@inheritDoc}
    */
@@ -337,11 +321,10 @@ public abstract class BaseTestMethod implements ITestNGMethod {
 
   /**
    * {@inheritDoc}
-   * @return the number of times this method or one of its clones must be invoked.
    */
   @Override
   public int getTotalInvocationCount() {
-    return 1;
+    return 0;
   }
 
   /**
@@ -435,8 +418,8 @@ public abstract class BaseTestMethod implements ITestNGMethod {
     // Init groups
     //
     {
-      ITestOrConfiguration annotation = getAnnotationFinder().findAnnotation(getMethod(), annotationClass);
-      ITestOrConfiguration classAnnotation = getAnnotationFinder().findAnnotation(getMethod().getDeclaringClass(), annotationClass);
+      ITestOrConfiguration annotation = getAnnotationFinder().findAnnotation(getConstructorOrMethod(), annotationClass);
+      ITestOrConfiguration classAnnotation = getAnnotationFinder().findAnnotation(getConstructorOrMethod().getDeclaringClass(), annotationClass);
 
       setGroups(getStringArray(null != annotation ? annotation.getGroups() : null,
           null != classAnnotation ? classAnnotation.getGroups() : null));
@@ -446,8 +429,8 @@ public abstract class BaseTestMethod implements ITestNGMethod {
     // Init groups depended upon
     //
     {
-      ITestOrConfiguration annotation = getAnnotationFinder().findAnnotation(getMethod(), annotationClass);
-      ITestOrConfiguration classAnnotation = getAnnotationFinder().findAnnotation(getMethod().getDeclaringClass(), annotationClass);
+      ITestOrConfiguration annotation = getAnnotationFinder().findAnnotation(getConstructorOrMethod(), annotationClass);
+      ITestOrConfiguration classAnnotation = getAnnotationFinder().findAnnotation(getConstructorOrMethod().getDeclaringClass(), annotationClass);
 
       Map<String, Set<String>> xgd = calculateXmlGroupDependencies(m_xmlTest);
       List<String> xmlGroupDependencies = Lists.newArrayList();
@@ -809,4 +792,9 @@ public abstract class BaseTestMethod implements ITestNGMethod {
 
     return result;
   }
+
+  @Override
+  public String getQualifiedName() {
+	return getRealClass().getName() + "." + getMethodName();
+  }
 }
