@@ -44,6 +44,7 @@ import org.testng.internal.reflect.MethodMatcherContext;
 import org.testng.internal.thread.ThreadExecutionException;
 import org.testng.internal.thread.ThreadUtil;
 import org.testng.internal.thread.graph.IWorker;
+import org.testng.util.Strings;
 import org.testng.xml.XmlClass;
 import org.testng.xml.XmlSuite;
 import org.testng.xml.XmlTest;
@@ -87,12 +88,14 @@ public class Invoker implements IInvoker {
   private static final Predicate<ITestNGMethod, IClass> SAME_CLASS = new SameClassNamePredicate();
 
   private void setClassInvocationFailure(Class<?> clazz, Object instance) {
-    Set<Object> instances = m_classInvocationResults.get( clazz );
-    if (instances == null) {
-      instances = Sets.newHashSet();
-      m_classInvocationResults.put(clazz, instances);
+    synchronized(m_classInvocationResults){
+       Set<Object> instances = m_classInvocationResults.get( clazz );
+       if (instances == null) {
+         instances = Sets.newHashSet();
+         m_classInvocationResults.put(clazz, instances);
+       }
+       instances.add(instance);
     }
-    instances.add(instance);
   }
 
   private void setMethodInvocationFailure(ITestNGMethod method, Object instance) {
@@ -167,7 +170,7 @@ public class Invoker implements IInvoker {
       }
 
       ITestResult testResult= new TestResult(testClass,
-                                             instance,
+          (instance != null ? instance : tm.getInstance()),
                                              tm,
                                              null,
                                              System.currentTimeMillis(),
@@ -181,7 +184,7 @@ public class Invoker implements IInvoker {
           inst = instance;
         }
         Class<?> objectClass= inst.getClass();
-        Method method= tm.getMethod();
+        ConstructorOrMethod method= tm.getConstructorOrMethod();
 
         // Only run the configuration if
         // - the test is enabled and
@@ -193,13 +196,14 @@ public class Invoker implements IInvoker {
           if (MethodHelper.isEnabled(configurationAnnotation)) {
 
             if (!confInvocationPassed(tm, currentTestMethod, testClass, instance) && !alwaysRun) {
+              log(3, "Skipping " + Utils.detailedMethodName(tm, true));
               handleConfigurationSkip(tm, testResult, configurationAnnotation, currentTestMethod, instance, suite);
               continue;
             }
 
             log(3, "Invoking " + Utils.detailedMethodName(tm, true));
 
-            Object[] parameters = Parameters.createConfigurationParameters(tm.getMethod(),
+            Object[] parameters = Parameters.createConfigurationParameters(tm.getConstructorOrMethod().getMethod(),
                 params,
                 parameterValues,
                 currentTestMethod,
@@ -209,15 +213,16 @@ public class Invoker implements IInvoker {
                 testMethodResult);
             testResult.setParameters(parameters);
 
-            Object newInstance = null != instance ? instance: inst;
-
             runConfigurationListeners(testResult, true /* before */);
 
-            invokeConfigurationMethod(newInstance, tm,
-              parameters, testResult);
+            Object newInstance;
+            if (instance == null || !tm.getConstructorOrMethod().getDeclaringClass().isAssignableFrom(instance.getClass())) {
+              newInstance = inst;
+            } else {
+              newInstance = instance;
+            }
+            invokeConfigurationMethod(newInstance, tm, parameters, testResult);
 
-            // TODO: probably we should trigger the event for each instance???
-            testResult.setEndMillis(System.currentTimeMillis());
             runConfigurationListeners(testResult, false /* after */);
           }
           else {
@@ -299,7 +304,7 @@ public class Invoker implements IInvoker {
       return;
     }
     Utils.log("", 3, "Failed to invoke configuration method "
-        + tm.getRealClass().getName() + "." + tm.getMethodName() + ":" + cause.getMessage());
+        + tm.getQualifiedName() + ":" + cause.getMessage());
     handleException(cause, tm, testResult, 1);
     runConfigurationListeners(testResult, false /* after */);
 
@@ -398,12 +403,14 @@ public class Invoker implements IInvoker {
    * @return true if this class or a parent class failed to initialize.
    */
   private boolean classConfigurationFailed(Class<?> cls) {
-    for (Class<?> c : m_classInvocationResults.keySet()) {
-      if (c == cls || c.isAssignableFrom(cls)) {
-        return true;
-      }
+    synchronized(m_classInvocationResults){
+       for (Class<?> c : m_classInvocationResults.keySet()) {
+         if (c == cls || c.isAssignableFrom(cls)) {
+           return true;
+         }
+       }
+       return false;
     }
-    return false;
   }
 
   /**
@@ -412,19 +419,20 @@ public class Invoker implements IInvoker {
    */
   private boolean confInvocationPassed(ITestNGMethod method, ITestNGMethod currentTestMethod,
       IClass testClass, Object instance) {
-    boolean result= true;
+    boolean result = true;
 
     Class<?> cls = testClass.getRealClass();
 
     if(m_suiteState.isFailed()) {
-      result= false;
-    }
-    else {
+      result = false;
+    } else {
       if (classConfigurationFailed(cls)) {
         if (! m_continueOnFailedConfiguration) {
           result = !classConfigurationFailed(cls);
         } else {
-          result = !m_classInvocationResults.get(cls).contains(instance);
+          synchronized(m_classInvocationResults){
+             result = !m_classInvocationResults.get(cls).contains(instance);
+          }
         }
       }
       // if method is BeforeClass, currentTestMethod will be null
@@ -434,22 +442,23 @@ public class Invoker implements IInvoker {
         result = !m_methodInvocationResults.get(currentTestMethod).contains(getMethodInvocationToken(currentTestMethod, instance));
       }
       else if (! m_continueOnFailedConfiguration) {
-        for(Class<?> clazz: m_classInvocationResults.keySet()) {
-//          if (clazz == cls) {
-          if(clazz.isAssignableFrom(cls)) {
-            result= false;
-            break;
-          }
+        synchronized(m_classInvocationResults){
+           for(Class<?> clazz : m_classInvocationResults.keySet()) {
+             if (clazz.isAssignableFrom(cls)) {
+               result = false;
+               break;
+             }
+           }
         }
       }
     }
 
     // check if there are failed @BeforeGroups
-    String[] groups= method.getGroups();
+    String[] groups = method.getGroups();
     if(null != groups && groups.length > 0) {
-      for(String group: groups) {
+      for(String group : groups) {
         if(m_beforegroupsFailures.containsKey(group)) {
-          result= false;
+          result = false;
           break;
         }
       }
@@ -459,8 +468,8 @@ public class Invoker implements IInvoker {
 
    // Creates a token for tracking a unique invocation of a method on an instance.
    // Is used when configFailurePolicy=continue.
-  private Object getMethodInvocationToken(ITestNGMethod method, Object instance) {
-    return String.format("%s+%d", instance.toString(), method.getCurrentInvocationCount());
+  private static Object getMethodInvocationToken(ITestNGMethod method, Object instance) {
+    return String.format("%s+%d+%d", instance.toString(), method.getCurrentInvocationCount(), method.getParameterInvocationCount());
   }
 
   /**
@@ -487,7 +496,6 @@ public class Invoker implements IInvoker {
     {
       InvokedMethod invokedMethod= new InvokedMethod(targetInstance,
                                           tm,
-                                          params,
                                           System.currentTimeMillis(),
                                           testResult);
 
@@ -495,16 +503,16 @@ public class Invoker implements IInvoker {
       m_notifier.addInvokedMethod(invokedMethod);
       try {
         Reporter.setCurrentTestResult(testResult);
-        Method method = tm.getMethod();
+        ConstructorOrMethod method = tm.getConstructorOrMethod();
 
         //
         // If this method is a IConfigurable, invoke its run() method
         //
         IConfigurable configurableInstance =
-          IConfigurable.class.isAssignableFrom(tm.getMethod().getDeclaringClass()) ?
+          IConfigurable.class.isAssignableFrom(method.getDeclaringClass()) ?
           (IConfigurable) targetInstance : m_configuration.getConfigurable();
         if (configurableInstance != null) {
-          MethodInvocationHelper.invokeConfigurable(targetInstance, params, configurableInstance, method,
+          MethodInvocationHelper.invokeConfigurable(targetInstance, params, configurableInstance, method.getMethod(),
               testResult);
         }
         else {
@@ -512,7 +520,7 @@ public class Invoker implements IInvoker {
           // Not a IConfigurable, invoke directly
           //
           if (MethodHelper.calculateTimeOut(tm) <= 0) {
-            MethodInvocationHelper.invokeMethod(method, targetInstance, params);
+            MethodInvocationHelper.invokeMethod(method.getMethod(), targetInstance, params);
           }
           else {
             MethodInvocationHelper.invokeWithTimeout(tm, targetInstance, params, testResult);
@@ -532,6 +540,7 @@ public class Invoker implements IInvoker {
         throw new TestNGException(ex);
       }
       finally {
+        testResult.setEndMillis(System.currentTimeMillis());
         Reporter.setCurrentTestResult(testResult);
         runInvokedMethodListeners(AFTER_INVOCATION, invokedMethod, testResult);
         Reporter.setCurrentTestResult(null);
@@ -592,6 +601,15 @@ public class Invoker implements IInvoker {
       suite, params, parameterValues,
       instance, testResult);
 
+    if (!confInvocationPassed(tm, tm, testClass, instance)) {
+      ITestResult result = registerSkippedTestResult(tm, instance, System.currentTimeMillis(),
+              getExceptionDetails(instance));
+      m_notifier.addSkippedTest(tm, result);
+      tm.incrementCurrentInvocationCount();
+
+      return result;
+    }
+
     //
     // Create the ExtraOutput for this method
     //
@@ -611,7 +629,6 @@ public class Invoker implements IInvoker {
 
       invokedMethod= new InvokedMethod(instance,
           tm,
-          parameterValues,
           System.currentTimeMillis(),
           testResult);
 
@@ -623,42 +640,36 @@ public class Invoker implements IInvoker {
         runTestListeners(testResult);
       }
 
+      log(3, "Invoking " + tm.getQualifiedName());
       runInvokedMethodListeners(BEFORE_INVOCATION, invokedMethod, testResult);
 
       m_notifier.addInvokedMethod(invokedMethod);
 
       Method thisMethod = tm.getConstructorOrMethod().getMethod();
 
-      if(confInvocationPassed(tm, tm, testClass, instance)) {
-        log(3, "Invoking " + tm.getRealClass().getName() + "." + tm.getMethodName());
+      // If this method is a IHookable, invoke its run() method
+      IHookable hookableInstance =
+          IHookable.class.isAssignableFrom(tm.getRealClass()) ?
+          (IHookable) instance : m_configuration.getHookable();
 
-        // If this method is a IHookable, invoke its run() method
-        IHookable hookableInstance =
-            IHookable.class.isAssignableFrom(tm.getRealClass()) ?
-            (IHookable) instance : m_configuration.getHookable();
-
-        if (MethodHelper.calculateTimeOut(tm) <= 0) {
-          if (hookableInstance != null) {
-            MethodInvocationHelper.invokeHookable(instance,
-                parameterValues, hookableInstance, thisMethod, testResult);
-          } else {
-            // Not a IHookable, invoke directly
-            MethodInvocationHelper.invokeMethod(thisMethod, instance,
-                parameterValues);
-          }
-          testResult.setStatus(ITestResult.SUCCESS);
+      if (MethodHelper.calculateTimeOut(tm) <= 0) {
+        if (hookableInstance != null) {
+          MethodInvocationHelper.invokeHookable(instance,
+              parameterValues, hookableInstance, thisMethod, testResult);
         } else {
-          // Method with a timeout
-          MethodInvocationHelper.invokeWithTimeout(tm, instance, parameterValues, testResult, hookableInstance);
+          // Not a IHookable, invoke directly
+          MethodInvocationHelper.invokeMethod(thisMethod, instance,
+              parameterValues);
         }
-      }
-      else {
-        testResult.setStatus(ITestResult.SKIP);
+        setTestStatus(testResult, ITestResult.SUCCESS);
+      } else {
+        // Method with a timeout
+        MethodInvocationHelper.invokeWithTimeout(tm, instance, parameterValues, testResult, hookableInstance);
       }
     }
     catch(InvocationTargetException ite) {
       testResult.setThrowable(ite.getCause());
-      testResult.setStatus(ITestResult.FAILURE);
+      setTestStatus(testResult, ITestResult.FAILURE);
     }
     catch(ThreadExecutionException tee) { // wrapper for TestNGRuntimeException
       Throwable cause= tee.getCause();
@@ -668,11 +679,11 @@ public class Invoker implements IInvoker {
       else {
         testResult.setThrowable(cause);
       }
-      testResult.setStatus(ITestResult.FAILURE);
+      setTestStatus(testResult, ITestResult.FAILURE);
     }
     catch(Throwable thr) { // covers the non-wrapper exceptions
       testResult.setThrowable(thr);
-      testResult.setStatus(ITestResult.FAILURE);
+      setTestStatus(testResult, ITestResult.FAILURE);
     }
     finally {
       // Set end time ASAP
@@ -729,6 +740,53 @@ public class Invoker implements IInvoker {
     return testResult;
   }
 
+  private static void setTestStatus(ITestResult result, int status) {
+    // set the test to success as long as the testResult hasn't been changed by the user via
+    // Reporter.getCurrentTestResult
+    if (result.getStatus() == ITestResult.STARTED) {
+      result.setStatus(status);
+    }
+  }
+
+  private Throwable getExceptionDetails(Object instance) {
+    Set<ITestResult> configResults = m_testContext.getFailedConfigurations().getAllResults();
+    if (configResults.isEmpty()) {
+      configResults = m_testContext.getSkippedConfigurations().getAllResults();
+    }
+    for (ITestResult configResult : configResults) {
+      if (sameInstance(configResult, instance)) {
+        return configResult.getThrowable();
+      }
+    }
+    if (configResults.isEmpty()) {
+      //if we are here it means we have a test method skip due to a @BeforeSuite failure in a different <test> maybe
+      //So we will have to find out that first failure/skip and get its throwable and pack that information into our
+      //current test method's test result.
+      return getConfigFailureException();
+    } else {
+      //If we are here it perhaps means that the test method is being skipped because there was a configuration
+      //failure in a different class due to @BeforeGroups being used.
+      //So lets just find the first exception information and then just pack it in.
+      return configResults.iterator().next().getThrowable();
+    }
+  }
+
+  private Throwable getConfigFailureException() {
+    Throwable t = null;
+    for (IInvokedMethod method : m_testContext.getSuite().getAllInvokedMethods()) {
+      ITestNGMethod m = method.getTestMethod();
+      if (m.isBeforeSuiteConfiguration() && (! method.getTestResult().isSuccess())) {
+        t = method.getTestResult().getThrowable();
+        break;
+      }
+    }
+    return t;
+  }
+
+  private boolean sameInstance(ITestResult configResult, Object instance) {
+    return (configResult.getInstance()!= null ) && (configResult.getInstance().equals(instance));
+  }
+
   void collectResults(ITestNGMethod testMethod, ITestResult result) {
       // Collect the results
       final int status = result.getStatus();
@@ -846,8 +904,7 @@ public class Invoker implements IInvoker {
         // don't pass the IClass or the instance as the method may be external
         // the invocation must be similar to @BeforeTest/@BeforeSuite
         invokeConfigurations(null, beforeMethodsArray, suite, params,
-            null, /* no parameter values */
-            null);
+            /* no parameter values */ null, instance);
       }
 
       //
@@ -908,8 +965,7 @@ public class Invoker implements IInvoker {
       // don't pass the IClass or the instance as the method may be external
       // the invocation must be similar to @BeforeTest/@BeforeSuite
       invokeConfigurations(null, afterMethodsArray, suite, params,
-          null, /* no parameter values */
-          null);
+          /* no parameter values */ null, instance);
 
       // Remove the groups so they don't get run again
       groupMethods.removeAfterGroups(filteredGroups.keySet());
@@ -1011,7 +1067,7 @@ public class Invoker implements IInvoker {
     assert null != testMethod.getTestClass()
         : "COULDN'T FIND TESTCLASS FOR " + testMethod.getRealClass();
 
-    if (!MethodHelper.isEnabled(testMethod.getMethod(), m_annotationFinder)) {
+    if (!MethodHelper.isEnabled(testMethod.getConstructorOrMethod().getMethod(), m_annotationFinder)) {
       // return if the method is not enabled. No need to do any more calculations
       return Collections.emptyList();
     }
@@ -1072,9 +1128,15 @@ public class Invoker implements IInvoker {
 
         if (bag.hasErrors()) {
           final ITestResult tr = bag.errorResult;
-          tr.setStatus(ITestResult.SKIP);
+          Throwable throwable = tr.getThrowable();
+          if (throwable instanceof TestNGException) {
+            tr.setStatus(ITestResult.FAILURE);
+            m_notifier.addFailedTest(testMethod, tr);
+          } else {
+            tr.setStatus(ITestResult.SKIP);
+            m_notifier.addSkippedTest(testMethod, tr);
+          }
           runTestListeners(tr);
-          m_notifier.addSkippedTest(testMethod, tr);
           result.add(tr);
           continue;
         }
@@ -1090,10 +1152,11 @@ public class Invoker implements IInvoker {
               Object[] next = allParameterValues.next();
               if (next == null) {
                 // skipped value
+                parametersIndex++;
                 continue;
               }
               Object[] parameterValues = injectParameters(next,
-                  testMethod.getMethod(), testContext, null /* test result */);
+                  testMethod.getConstructorOrMethod().getMethod(), testContext, null /* test result */);
 
               TestMethodWithDataProviderMethodWorker w =
                 new TestMethodWithDataProviderMethodWorker(this,
@@ -1118,13 +1181,14 @@ public class Invoker implements IInvoker {
               Object[] next = allParameterValues.next();
               if (next == null) {
                 // skipped value
+                parametersIndex++;
                 continue;
               }
               Object[] parameterValues = injectParameters(next,
-                  testMethod.getMethod(), testContext, null /* test result */);
+                  testMethod.getConstructorOrMethod().getMethod(), testContext, null /* test result */);
 
               List<ITestResult> tmpResults = Lists.newArrayList();
-
+              int tmpResultsIndex = -1;
               try {
                 tmpResults.add(invokeTestMethod(instance,
                     testMethod,
@@ -1136,21 +1200,24 @@ public class Invoker implements IInvoker {
                     beforeMethods,
                     afterMethods,
                     groupMethods, failure));
+                tmpResultsIndex++;
               }
               finally {
-                if (failure.instances.isEmpty()) {
+              	boolean lastSucces = false;
+                if (tmpResultsIndex >= 0) {
+                  lastSucces = (tmpResults.get(tmpResultsIndex).getStatus() == ITestResult.SUCCESS);
+                }
+                if (failure.instances.isEmpty() || lastSucces) {
                   result.addAll(tmpResults);
                 } else {
-                  for (Object failedInstance : failure.instances) {
                     List<ITestResult> retryResults = Lists.newArrayList();
 
                     failure.count = retryFailed(
-                            failedInstance, testMethod, suite, testClass, beforeMethods,
+                    		instance, testMethod, suite, testClass, beforeMethods,
                      afterMethods, groupMethods, retryResults,
                      failure.count, expectedExceptionHolder,
                      testContext, parameters, parametersIndex);
                   result.addAll(retryResults);
-                  }
                 }
 
                 //
@@ -1201,6 +1268,7 @@ public class Invoker implements IInvoker {
         System.currentTimeMillis(),
         m_testContext);
     result.setStatus(TestResult.SKIP);
+    Reporter.setCurrentTestResult(result);
     runTestListeners(result);
 
     return result;
@@ -1243,12 +1311,16 @@ public class Invoker implements IInvoker {
             new Parameters.MethodParameters(parameters,
                 testMethod.findMethodParameters(testContext.getCurrentXmlTest()),
                 parameterValues,
-                testMethod.getMethod(), testContext, testResult),
+                testMethod.getConstructorOrMethod().getMethod(), testContext, testResult),
             suite,
             m_annotationFinder,
             fedInstance));
     }
     catch(Throwable cause) {
+      String msg = Utils.longStackTrace(cause.getCause() != null ? cause.getCause() : cause, true);
+      if (Strings.isNotNullAndNotEmpty(msg) ) {
+        Utils.error(msg);
+      }
       return new ParameterBag(
           new TestResult(
               testMethod.getTestClass(),
@@ -1354,7 +1426,10 @@ public class Invoker implements IInvoker {
       if (willRetry) {
         resultsToRetry.add(testResult);
         failure.count++;
-        failure.instances.add(testResult.getInstance());
+        Object instance = testResult.getInstance();
+        if (!failure.instances.contains(instance)) {
+          failure.instances.add(instance);
+        }
         testResult.setStatus(ITestResult.SKIP);
       } else {
         testResult.setStatus(status);
@@ -1410,7 +1485,7 @@ public class Invoker implements IInvoker {
       }
     }
 
-    ThreadUtil.execute(workers, threadPoolSize, maxTimeOut, true);
+    ThreadUtil.execute("methods", workers, threadPoolSize, maxTimeOut, true);
 
     //
     // Collect all the TestResults
@@ -1588,7 +1663,7 @@ public class Invoker implements IInvoker {
     List<ITestNGMethod> vResult= Lists.newArrayList();
 
     for(ITestNGMethod tm : methods) {
-      if (predicate.isTrue(tm, testClass)) {
+      if (predicate.isTrue(tm, testClass) &&  (!TestNgMethodUtils.containsConfigurationMethod(tm, vResult)) ) {
         log(10, "Keeping method " + tm + " for class " + testClass);
         vResult.add(tm);
       } else {
@@ -1642,32 +1717,34 @@ public class Invoker implements IInvoker {
   // TODO: move this from here as it is directly called from TestNG
   public static void runTestListeners(ITestResult tr, List<ITestListener> listeners) {
     for (ITestListener itl : listeners) {
-      switch(tr.getStatus()) {
-        case ITestResult.SKIP: {
-          itl.onTestSkipped(tr);
-          break;
-        }
-        case ITestResult.SUCCESS_PERCENTAGE_FAILURE: {
-          itl.onTestFailedButWithinSuccessPercentage(tr);
-          break;
-        }
-        case ITestResult.FAILURE: {
-          itl.onTestFailure(tr);
-          break;
-        }
-        case ITestResult.SUCCESS: {
-          itl.onTestSuccess(tr);
-          break;
-        }
-
-        case ITestResult.STARTED: {
-          itl.onTestStart(tr);
-          break;
-        }
-
-        default: {
-          assert false : "UNKNOWN STATUS:" + tr;
+      try {
+        switch(tr.getStatus()) {
+          case ITestResult.SKIP: {
+            itl.onTestSkipped(tr);
+            break;
+          }
+          case ITestResult.SUCCESS_PERCENTAGE_FAILURE: {
+            itl.onTestFailedButWithinSuccessPercentage(tr);
+            break;
+          }
+          case ITestResult.FAILURE: {
+            itl.onTestFailure(tr);
+            break;
+          }
+          case ITestResult.SUCCESS: {
+            itl.onTestSuccess(tr);
+            break;
+          }
+          case ITestResult.STARTED: {
+            itl.onTestStart(tr);
+            break;
+          }
+          default: {
+            assert false : "UNKNOWN STATUS:" + tr;
+          }
         }
+      } catch (Exception e) {
+        e.printStackTrace();
       }
     }
   }
