@@ -1,26 +1,5 @@
 package org.testng.internal;
 
-import java.io.BufferedReader;
-import java.io.BufferedWriter;
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileOutputStream;
-import java.io.FileWriter;
-import java.io.IOException;
-import java.io.InputStream;
-import java.io.OutputStream;
-import java.io.OutputStreamWriter;
-import java.io.PrintWriter;
-import java.io.StringReader;
-import java.io.StringWriter;
-import java.lang.reflect.Method;
-import java.lang.reflect.Modifier;
-import java.util.Arrays;
-import java.util.HashMap;
-import java.util.List;
-import java.util.Map;
-import java.util.StringTokenizer;
-
 import org.testng.ITestNGMethod;
 import org.testng.TestNG;
 import org.testng.TestNGException;
@@ -30,10 +9,15 @@ import org.testng.annotations.ITestAnnotation;
 import org.testng.collections.Lists;
 import org.testng.internal.annotations.AnnotationHelper;
 import org.testng.internal.annotations.IAnnotationFinder;
-import org.testng.log.TextFormatter;
+import org.testng.log4testng.Logger;
 import org.testng.reporters.XMLStringBuffer;
 import org.testng.xml.XmlClass;
 
+import java.io.*;
+import java.lang.reflect.Method;
+import java.lang.reflect.Modifier;
+import java.util.*;
+
 /**
  * Helper methods to parse annotations.
  *
@@ -46,6 +30,20 @@ public final class Utils {
       {'*','/','\\','?','%',':',';','<','>','&','~','|'};
   public static final char CHAR_REPLACEMENT = '_';
   public static final char UNICODE_REPLACEMENT = 0xFFFD;
+  private static final String FORMAT = String.format("[%s]",Utils.class.getSimpleName());
+
+  private static final Logger LOG = Logger.getLogger(Utils.class);
+
+  private static final Map<Character, String> ESCAPES = new HashMap<Character, String>() {
+    private static final long serialVersionUID = 1285607660247157523L;
+
+    {
+      put('<', "&lt;");
+      put('>', "&gt;");
+      put('\'', "&apos;");
+      put('"', "&quot;");
+      put('&', "&amp;");
+    }};
 
   /**
    * Hide constructor for utility class.
@@ -107,14 +105,14 @@ public final class Utils {
       if (!file.exists()) {
         file.createNewFile();
       }
-      final OutputStreamWriter w = new OutputStreamWriter(new FileOutputStream(file), "UTF-8");
-      if (prefix != null) {
-        w.append(prefix);
+      try (final OutputStreamWriter w = new OutputStreamWriter(new FileOutputStream(file), "UTF-8")) {
+        if (prefix != null) {
+          w.append(prefix);
+        }
+        xsb.toWriter(w);
       }
-      xsb.toWriter(w);
-      w.close();
     } catch(IOException ex) {
-      ex.printStackTrace();
+      LOG.error(ex.getMessage(), ex);
     }
   }
 
@@ -167,18 +165,18 @@ public final class Utils {
       File outputFile = new File(outDir, fileName);
       if (!append) {
         outputFile.delete();
-        log("Attempting to create " + outputFile);
-        log("  Directory " + outDir + " exists: " + outDir.exists());
+        log(FORMAT, 3, "Attempting to create " + outputFile);
+        log(FORMAT, 3, "  Directory " + outDir + " exists: " + outDir.exists());
         outputFile.createNewFile();
       }
       writeFile(outputFile, sb, encoding, append);
     }
     catch (IOException e) {
       if (TestRunner.getVerbose() > 1) {
-        e.printStackTrace();
+        LOG.error(e.getMessage(),e);
       }
       else {
-        log("[Utils]", 1, e.getMessage());
+        log(FORMAT, 1, e.getMessage());
       }
     }
   }
@@ -193,11 +191,10 @@ public final class Utils {
     }
     catch(IOException ex) {
       if (TestRunner.getVerbose() > 1) {
-        System.err.println("ERROR WHILE WRITING TO " + outputFile);
-        ex.printStackTrace();
+        LOG.error("ERROR WHILE WRITING TO " + outputFile,ex);
       }
       else {
-        log("[Utils]", 1, "Error while writing to " + outputFile + ": " + ex.getMessage());
+        log(FORMAT, 1, "Error while writing to " + outputFile + ": " + ex.getMessage());
       }
     }
     finally {
@@ -236,7 +233,7 @@ public final class Utils {
     if (!outputFile.exists()) {
       outputFile.createNewFile();
     }
-    OutputStreamWriter osw= null;
+    OutputStreamWriter osw;
     if (null != encoding) {
       osw = new OutputStreamWriter(new FileOutputStream(outputFile, append), encoding);
     }
@@ -254,11 +251,11 @@ public final class Utils {
    * @param result
    */
   public static void dumpMap(Map<?, ?> result) {
-    System.out.println("vvvvv");
+    LOG.info("vvvvv");
     for (Map.Entry<?, ?> entry : result.entrySet()) {
-      System.out.println(entry.getKey() + " => " + entry.getValue());
+      LOG.info(entry.getKey() + " => " + entry.getValue());
     }
-    System.out.println("^^^^^");
+    LOG.info("^^^^^");
   }
 
   /**
@@ -292,7 +289,6 @@ public final class Utils {
     if (null != tm) {
       String[] groups = tm.getDependsOnGroups();
 
-      //       ppp("Method:" + m + " #Groups:" + groups.length);
       for (String group : groups) {
         vResult.add(group);
       }
@@ -322,7 +318,6 @@ public final class Utils {
     if (null != tm) {
       String[] groups = tm.getGroups();
 
-      //       ppp("Method:" + m + " #Groups:" + groups.length);
       for (String group : groups) {
         vResult.add(group);
       }
@@ -383,43 +378,25 @@ public final class Utils {
     // Why this coupling on a static member of TestRunner.getVerbose()?
     if (TestRunner.getVerbose() >= level) {
       if (cls.length() > 0) {
-        System.out.println("[" + cls + "] " + msg);
+        LOG.info("[" + cls + "] " + msg);
       }
       else {
-        System.out.println(msg);
+        LOG.info(msg);
       }
     }
   }
 
   public static void error(String errorMessage) {
-    System.err.println("[Error] " + errorMessage);
+    LOG.error("[Error] " + errorMessage);
   }
 
-  /**
-   * @return The number of methods invoked, taking into account the number
-   * of instances.
-   */
-//  public static int calculateInvokedMethodCount(IResultMap map) {
-//    return calculateInvokedMethodCount(
-//        (ITestNGMethod[]) map.getAllMethods().toArray(new ITestNGMethod[map.size()]));
-//  }
-
   public static int calculateInvokedMethodCount(ITestNGMethod[] methods) {
     return methods.length;
-//    int result = 0;
-//
-//    for (ITestNGMethod method : methods) {
-//      int instanceCount = method.getInvocationCount();
-//      result += instanceCount;
-//    }
-//
-//    return result;
   }
 
-//  public static int calculateInvokedMethodCount(Map<ITestNGMethod, ITestResult> methods) {
-//    return calculateInvokedMethodCount(methods.keySet().toArray(new ITestNGMethod[methods.values()
-//                                                                .size()]));
-//  }
+  public static int calculateInvokedMethodCount(List<ITestNGMethod> methods) {
+    return methods.size();
+  }
 
   /**
    * Tokenize the string using the separator.
@@ -450,13 +427,13 @@ public final class Utils {
 
   public static void logInvocation(String reason, Method thisMethod, Object[] parameters) {
     String clsName = thisMethod.getDeclaringClass().getName();
-    int n = clsName.lastIndexOf(".");
+    int n = clsName.lastIndexOf('.');
     if (n >= 0) {
       clsName = clsName.substring(n + 1);
     }
     String methodName = clsName + '.' + thisMethod.getName();
     if (TestRunner.getVerbose() >= 2) {
-      StringBuffer paramString = new StringBuffer();
+      StringBuilder paramString = new StringBuilder();
       if (parameters != null) {
         for (Object p : parameters) {
           paramString.append(p.toString()).append(' ');
@@ -469,26 +446,19 @@ public final class Utils {
   public static void writeResourceToFile(File file, String resourceName, Class<?> clasz) throws IOException {
     InputStream inputStream = clasz.getResourceAsStream("/" + resourceName);
     if (inputStream == null) {
-      System.err.println("Couldn't find resource on the class path: " + resourceName);
-//      throw new IllegalArgumentException("Resource does not exist: " + resourceName);
+      LOG.error("Couldn't find resource on the class path: " + resourceName);
+      return;
     }
-
-    else {
-
-      try {
-        FileOutputStream outputStream = new FileOutputStream(file);
-        try {
-          int nread;
-          byte[] buffer = new byte[4096];
-          while (0 < (nread = inputStream.read(buffer))) {
-            outputStream.write(buffer, 0, nread);
-          }
-        } finally {
-          outputStream.close();
+    try {
+      try (FileOutputStream outputStream = new FileOutputStream(file)) {
+        int nread;
+        byte[] buffer = new byte[4096];
+        while (0 < (nread = inputStream.read(buffer))) {
+          outputStream.write(buffer, 0, nread);
         }
-      } finally {
-        inputStream.close();
       }
+    } finally {
+      inputStream.close();
     }
   }
 
@@ -514,42 +484,63 @@ public final class Utils {
 
   /**
    * @return an array of two strings: the short stack trace and the long stack trace.
+   * @deprecated - Please consider using :
+   * <ul>
+   *     <li>{@link Utils#longStackTrace(Throwable, boolean)} - for getting full stack trace</li>
+   *     <li>{@link Utils#shortStackTrace(Throwable, boolean)} - for getting short stack trace</li>
+   * </ul>
    */
+  @Deprecated
   public static String[] stackTrace(Throwable t, boolean toHtml) {
+    return new String[] {
+        shortStackTrace(t, toHtml), longStackTrace(t, toHtml)
+    };
+  }
+
+  /**
+   * Helper that returns a short stack trace.
+   * @param t - The {@link Throwable} exception
+   * @param toHtml - <code>true</code> if the stacktrace should be translated to html as well
+   * @return - A string that represents the short stack trace.
+   */
+  public static String longStackTrace(Throwable t, boolean toHtml) {
+    return buildStrackTrace(t, toHtml, StackTraceType.FULL);
+  }
+
+  /**
+   * Helper that returns a long stack trace.
+   * @param t - The {@link Throwable} exception
+   * @param toHtml - <code>true</code> if the stacktrace should be translated to html as well
+   * @return - A string that represents the full stack trace.
+   */
+  public static String shortStackTrace(Throwable t, boolean toHtml) {
+    return buildStrackTrace(t, toHtml, StackTraceType.SHORT);
+  }
+
+  private static String buildStrackTrace(Throwable t, boolean toHtml, StackTraceType type) {
     StringWriter sw = new StringWriter();
     PrintWriter pw = new PrintWriter(sw);
     t.printStackTrace(pw);
     pw.flush();
-
-    String fullStackTrace = sw.getBuffer().toString();
-    String shortStackTrace;
-
-    if (Boolean.getBoolean(TestNG.SHOW_TESTNG_STACK_FRAMES) || TestRunner.getVerbose() >= 2) {
-      shortStackTrace = fullStackTrace;
-    } else {
-      shortStackTrace = filterTrace(sw.getBuffer().toString());
+    String stackTrace = sw.getBuffer().toString();
+    if (type == StackTraceType.SHORT && !isTooVerbose()) {
+      stackTrace = filterTrace(sw.getBuffer().toString());
     }
-
     if (toHtml) {
-      shortStackTrace = escapeHtml(shortStackTrace);
-      fullStackTrace = escapeHtml(fullStackTrace);
+      stackTrace = escapeHtml(stackTrace);
     }
-
-    return new String[] {
-        shortStackTrace, fullStackTrace
-    };
+    return stackTrace;
   }
 
-  private static final Map<Character, String> ESCAPES = new HashMap<Character, String>() {
-    private static final long serialVersionUID = 1285607660247157523L;
+  private static boolean isTooVerbose() {
+    return Boolean.getBoolean(TestNG.SHOW_TESTNG_STACK_FRAMES) || TestRunner.getVerbose() >= 2;
+  }
+
+
+  private enum StackTraceType {
+    SHORT,FULL
+  }
 
-  {
-    put('<', "&lt;");
-    put('>', "&gt;");
-    put('\'', "&apos;");
-    put('"', "&quot;");
-    put('&', "&amp;");
-  }};
 
   public static String escapeHtml(String s) {
     if (s == null) {
@@ -580,17 +571,17 @@ public final class Utils {
 
     for (int i = 0; i < s.length(); i++) {
       char c = s.charAt(i);
-      char ca = (Character.isDefined(c)) ? c: UNICODE_REPLACEMENT;
+      char ca = Character.isDefined(c) ? c: UNICODE_REPLACEMENT;
       result.append(ca);
     }
 
     return result.toString();
   }
 
-  private static String filterTrace(String trace) {
+  static String filterTrace(String trace) {
     StringReader   stringReader = new StringReader(trace);
     BufferedReader bufferedReader = new BufferedReader(stringReader);
-    StringBuffer buf = new StringBuffer();
+    StringBuilder buf = new StringBuilder();
 
     try {
       // first line contains the thrown exception
@@ -606,6 +597,7 @@ public final class Utils {
       String[] excludedStrings = new String[] {
           "org.testng",
           "reflect",
+          "org.gradle",
           "org.apache.maven.surefire"
       };
 
@@ -624,7 +616,7 @@ public final class Utils {
         }
       }
       if (excludedCount > 0) {
-        buf.append("... Removed " + excludedCount + " stack frames");
+        buf.append("... Removed ").append(excludedCount).append(" stack frames");
       }
     }
     catch(IOException ioex) {
@@ -651,43 +643,54 @@ public final class Utils {
   }
 
   public static String detailedMethodName(ITestNGMethod method, boolean fqn) {
-    StringBuffer buf= new StringBuffer();
-    if(method.isBeforeSuiteConfiguration()) {
-      buf.append("@BeforeSuite ");
-    }
-    else if(method.isBeforeTestConfiguration()) {
-      buf.append("@BeforeTest ");
-    }
-    else if(method.isBeforeClassConfiguration()) {
-      buf.append("@BeforeClass ");
-    }
-    else if(method.isBeforeGroupsConfiguration()) {
-      buf.append("@BeforeGroups ");
-    }
-    else if(method.isBeforeMethodConfiguration()) {
-      buf.append("@BeforeMethod ");
-    }
-    else if(method.isAfterMethodConfiguration()) {
-      buf.append("@AfterMethod ");
-    }
-    else if(method.isAfterGroupsConfiguration()) {
-      buf.append("@AfterGroups ");
-    }
-    else if(method.isAfterClassConfiguration()) {
-      buf.append("@AfterClass ");
-    }
-    else if(method.isAfterTestConfiguration()) {
-      buf.append("@AfterTest ");
-    }
-    else if(method.isAfterSuiteConfiguration()) {
-      buf.append("@AfterSuite ");
+    String tempName = annotationFormFor(method);
+    if (! tempName.isEmpty()) {
+      tempName += " ";
     }
+    return tempName + (fqn ? method.toString() : method.getMethodName());
+  }
 
-    return buf.append(fqn ? method.toString() : method.getMethodName()).toString();
+  /**
+   * Given a TestNG method, returns the corresponding annotation based on the method type
+   * @param method - An {@link ITestNGMethod} object.
+   * @return - A String representation of the corresponding annotation.
+   */
+  public static String annotationFormFor(ITestNGMethod method) {
+    if(method.isBeforeSuiteConfiguration()) {
+      return "@BeforeSuite";
+    }
+    if(method.isBeforeTestConfiguration()) {
+      return "@BeforeTest";
+    }
+    if(method.isBeforeClassConfiguration()) {
+      return "@BeforeClass";
+    }
+    if(method.isBeforeGroupsConfiguration()) {
+      return "@BeforeGroups";
+    }
+    if(method.isBeforeMethodConfiguration()) {
+      return "@BeforeMethod";
+    }
+    if(method.isAfterMethodConfiguration()) {
+      return "@AfterMethod";
+    }
+    if(method.isAfterGroupsConfiguration()) {
+      return "@AfterGroups";
+    }
+    if(method.isAfterClassConfiguration()) {
+      return "@AfterClass";
+    }
+    if (method.isAfterTestConfiguration()) {
+      return "@AfterTest";
+    }
+    if(method.isAfterSuiteConfiguration()) {
+      return "@AfterSuite";
+    }
+    return "";
   }
 
   public static String arrayToString(String[] strings) {
-    StringBuffer result = new StringBuffer("");
+    StringBuilder result = new StringBuilder("");
     if ((strings != null) && (strings.length > 0)) {
       for (int i = 0; i < strings.length; i++) {
         result.append(strings[i]);
@@ -709,7 +712,6 @@ public final class Utils {
    *
    * @param fileName file name that could contain special characters.
    * @return fileName with special characters replaced
-   * @author Borojevic
    */
   public static String replaceSpecialCharacters(String fileName) {
    if (fileName == null || fileName.length() == 0) {
@@ -742,7 +744,7 @@ public final class Utils {
         out.write(buf, 0, len);
       }
     } catch(IOException e){
-      e.printStackTrace();
+      LOG.error(e.getMessage(),e);
     }
   }
 
@@ -758,9 +760,9 @@ public final class Utils {
       result.deleteOnExit();
 
       // Write to temp file
-      BufferedWriter out = new BufferedWriter(new FileWriter(result));
-      out.write(content);
-      out.close();
+      try (BufferedWriter out = new BufferedWriter(new FileWriter(result))) {
+        out.write(content);
+      }
 
       return result;
     } catch (IOException e) {
