@@ -32,7 +32,7 @@ import org.testng.xml.XmlSuite;
  * @author Abraham Lin
  */
 public class EmailableReporter2 implements IReporter {
-    private static final Logger LOG = Logger.getLogger(EmailableReporter.class);
+    private static final Logger LOG = Logger.getLogger(EmailableReporter2.class);
 
     protected PrintWriter writer;
 
@@ -41,9 +41,20 @@ public class EmailableReporter2 implements IReporter {
     // Reusable buffer
     private StringBuilder buffer = new StringBuilder();
 
+    private String fileName = "emailable-report.html";
+
+    private static final String JVM_ARG = "emailable.report2.name";
+
+    public void setFileName(String fileName) {
+        this.fileName = fileName;
+    }
+
+    public String getFileName() {
+        return fileName;
+    }
+
     @Override
-    public void generateReport(List<XmlSuite> xmlSuites, List<ISuite> suites,
-            String outputDirectory) {
+    public void generateReport(List<XmlSuite> xmlSuites, List<ISuite> suites, String outputDirectory) {
         try {
             writer = createWriter(outputDirectory);
         } catch (IOException e) {
@@ -64,8 +75,11 @@ public class EmailableReporter2 implements IReporter {
 
     protected PrintWriter createWriter(String outdir) throws IOException {
         new File(outdir).mkdirs();
-        return new PrintWriter(new BufferedWriter(new FileWriter(new File(
-                outdir, "emailable-report.html"))));
+        String jvmArg = System.getProperty(JVM_ARG);
+        if (jvmArg != null && !jvmArg.trim().isEmpty()) {
+            fileName = jvmArg;
+        }
+        return new PrintWriter(new BufferedWriter(new FileWriter(new File(outdir, fileName))));
     }
 
     protected void writeDocumentStart() {
@@ -196,7 +210,7 @@ public class EmailableReporter2 implements IReporter {
      * Writes a summary of all the test scenarios.
      */
     protected void writeScenarioSummary() {
-        writer.print("<table>");
+        writer.print("<table id='summary'>");
         writer.print("<thead>");
         writer.print("<tr>");
         writer.print("<th>Class</th>");
@@ -474,10 +488,18 @@ public class EmailableReporter2 implements IReporter {
         writer.print("<div class=\"messages\">");
         Iterator<String> iterator = reporterMessages.iterator();
         assert iterator.hasNext();
-        writer.print(Utils.escapeHtml(iterator.next()));
+        if (Reporter.getEscapeHtml()) {
+        	writer.print(Utils.escapeHtml(iterator.next()));
+        } else {
+        	writer.print(iterator.next());
+        }
         while (iterator.hasNext()) {
             writer.print("<br/>");
-            writer.print(Utils.escapeHtml(iterator.next()));
+            if (Reporter.getEscapeHtml()) {
+            	writer.print(Utils.escapeHtml(iterator.next()));
+            } else {
+            	writer.print(iterator.next());
+            }
         }
         writer.print("</div>");
     }
