@@ -190,8 +190,8 @@ public class XmlSuite implements Serializable, Cloneable {
 
   private static final long serialVersionUID = 4999962288272750226L;
 
-  public static final String DEFAULT_PRESERVE_ORDER = "true";
-  private String m_preserveOrder = DEFAULT_PRESERVE_ORDER;
+  public static final Boolean DEFAULT_PRESERVE_ORDER = Boolean.TRUE;
+  private Boolean m_preserveOrder = DEFAULT_PRESERVE_ORDER;
 
   private List<String> m_includedGroups = Lists.newArrayList();
   private List<String> m_excludedGroups = Lists.newArrayList();
@@ -699,7 +699,7 @@ public class XmlSuite implements Serializable, Cloneable {
     result.setConfigFailurePolicy(getConfigFailurePolicy());
     result.setThreadCount(getThreadCount());
     result.setDataProviderThreadCount(getDataProviderThreadCount());
-    result.setParameters(getAllParameters());
+    result.setParameters(getParameters());
     result.setVerbose(getVerbose());
     result.setXmlPackages(getXmlPackages());
 //    result.setBeanShellExpression(getExpression());
@@ -1012,11 +1012,19 @@ public class XmlSuite implements Serializable, Cloneable {
     return result;
   }
 
+  /**
+   * @deprecated Use {@link #setPreserveOrder(Boolean)} instead
+   */
+  @Deprecated
   public void setPreserveOrder(String f) {
+    setPreserveOrder(Boolean.valueOf(f));
+  }
+
+  public void setPreserveOrder(Boolean f) {
     m_preserveOrder = f;
   }
 
-  public String getPreserveOrder() {
+  public Boolean getPreserveOrder() {
     return m_preserveOrder;
   }
 
@@ -1025,7 +1033,9 @@ public class XmlSuite implements Serializable, Cloneable {
    * Note: do not modify the returned value, use {@link #addIncludedGroup(String)}.
    */
   public List<String> getIncludedGroups() {
-    if (m_xmlGroups != null) {
+    if (m_parentSuite != null) {
+      return m_parentSuite.getIncludedGroups();
+    } else if (m_xmlGroups != null && (m_xmlGroups.getRun() != null)) {
       return m_xmlGroups.getRun().getIncludes();
     } else {
       // deprecated
@@ -1056,7 +1066,9 @@ public class XmlSuite implements Serializable, Cloneable {
    * Note: do not modify the returned value, use {@link #addExcludedGroup(String)}.
    */
   public List<String> getExcludedGroups() {
-    if (m_xmlGroups != null) {
+    if (m_parentSuite != null) {
+      return m_parentSuite.getExcludedGroups();
+    } else if (m_xmlGroups != null && (m_xmlGroups.getRun() != null)) {
       return m_xmlGroups.getRun().getExcludes();
     } else {
       return m_excludedGroups;
