@@ -394,54 +394,6 @@ public class BaseTest extends BaseDistributedTest {
     verifyTests("Skipped", expectedSkipped, getSkippedTests());
   }
 
-  /**
-     *
-     * @param fileName The filename to parse
-     * @param regexp The regular expression
-     * @param resultLines An out parameter that will contain all the lines
-     * that matched the regexp
-     * @return A List<Integer> containing the lines of all the matches
-     *
-     * Note that the size() of the returned valuewill always be equal to
-     * result.size() at the end of this function.
-     */
-    public static List<Integer> grep(File fileName, String regexp, List<String> resultLines) {
-      List<Integer> resultLineNumbers = new ArrayList<>();
-      BufferedReader fr = null;
-      try {
-        fr = new BufferedReader(new FileReader(fileName));
-        String line = fr.readLine();
-        int currentLine = 0;
-        Pattern p = Pattern.compile(".*" + regexp + ".*");
-
-        while(null != line) {
-  //        ppp("COMPARING " + p + " TO @@@" + line + "@@@");
-          if(p.matcher(line).matches()) {
-            resultLines.add(line);
-            resultLineNumbers.add(currentLine);
-          }
-
-          line = fr.readLine();
-          currentLine++;
-        }
-      } catch(IOException e) {
-        e.printStackTrace();
-      }
-      finally {
-        if(null != fr) {
-          try {
-            fr.close();
-          }
-          catch(IOException ex) {
-            ex.printStackTrace();
-          }
-        }
-      }
-
-      return resultLineNumbers;
-
-    }
-
   private static class InternalTestRunnerFactory implements ITestRunnerFactory {
     private final BaseTest m_baseTest;
 
@@ -471,27 +423,6 @@ public class BaseTest extends BaseDistributedTest {
     }
   }
 
-  /**
-   *  Deletes all files and subdirectories under dir.
-
-   *  @return true if all deletions were successful.
-   *  If a deletion fails, the method stops attempting to delete and returns false.
-   */
-  public static boolean deleteDir(File dir) {
-    if (dir.isDirectory()) {
-      String[] children = dir.list();
-      for (String element : children) {
-        boolean success = deleteDir(new File(dir, element));
-        if (!success) {
-          return false;
-        }
-      }
-    }
-
-    // The directory is now empty so delete it
-    return dir.delete();
-  }
-
   protected void runTest(String cls, String[] passed, String[] failed, String[] skipped) {
     addClass(cls);
     run();
