@@ -12,8 +12,6 @@ public class FieldInjectionProvider {
 
   @DataProvider(name = "injection")
   public Object[][] create() {
-    return new Object[][] {
-        new Object[] { value },
-    };
+    return new Object[][]{{value}};
   }
 }
