@@ -3,13 +3,12 @@ package test.dataprovider;
 import org.testng.annotations.Factory;
 
 public class TestInstanceFactory {
+
   @Factory
   public Object[] init() {
     return new Object[] {
-        new TestInstanceTest(1),
-        new TestInstanceTest(2)
+        new TestInstanceSample(1),
+        new TestInstanceSample(2)
     };
   }
-
-
 }
