@@ -1,6 +1,7 @@
 package test.factory;
 
 import org.testng.Assert;
+import org.testng.ITestNGListener;
 import org.testng.TestListenerAdapter;
 import org.testng.TestNG;
 import org.testng.annotations.Test;
@@ -11,8 +12,11 @@ public class DisabledFactoryTest extends SimpleBaseTest {
 
   @Test
   public void disabledFactoryShouldNotRun() {
-    TestNG tng = create(DisabledFactorySampleTest.class);
+    TestNG tng = create(DisabledFactory.class);
+
     TestListenerAdapter tla = new TestListenerAdapter();
+    tng.addListener((ITestNGListener) tla);
+
     tng.run();
 
     Assert.assertEquals(tla.getPassedTests().size(), 0);
