@@ -1,16 +1,17 @@
 package test.listeners;
 
-import org.testng.Assert;
-import org.testng.ITestNGListener;
-import org.testng.TestListenerAdapter;
-import org.testng.TestNG;
+import org.testng.*;
 import org.testng.annotations.BeforeMethod;
 import org.testng.annotations.Test;
 import org.testng.collections.Lists;
 
+import org.testng.xml.XmlSuite;
 import test.SimpleBaseTest;
+import test.listeners.github956.ListenerFor956;
+import test.listeners.github956.TestClassContainer;
 
 import java.util.Arrays;
+import java.util.List;
 
 import static org.assertj.core.api.Assertions.assertThat;
 import static org.assertj.core.api.Assertions.entry;
@@ -121,9 +122,9 @@ public class ListenerTest extends SimpleBaseTest {
     MyClassListener.names.clear();
     TestNG tng = create(Derived1.class, Derived2.class);
     MyClassListener listener = new MyClassListener();
-    tng.addListener((Object)listener);
+    tng.addListener((ITestNGListener) listener);
     TestListenerAdapter adapter = new TestListenerAdapter();
-    tng.addListener(adapter);
+    tng.addListener((ITestNGListener) adapter);
     tng.run();
     assertThat(adapter.getFailedTests()).isEmpty();
     assertThat(adapter.getSkippedTests()).isEmpty();
@@ -141,9 +142,9 @@ public class ListenerTest extends SimpleBaseTest {
     MyClassListener.names.clear();
     TestNG tng = create(Derived3.class);
     MyClassListener listener = new MyClassListener();
-    tng.addListener((Object)listener);
+    tng.addListener((ITestNGListener) listener);
     TestListenerAdapter adapter = new TestListenerAdapter();
-    tng.addListener(adapter);
+    tng.addListener((ITestNGListener) adapter);
     tng.run();
     assertThat(adapter.getFailedTests()).isEmpty();
     assertThat(adapter.getSkippedTests()).isEmpty();
@@ -160,7 +161,7 @@ public class ListenerTest extends SimpleBaseTest {
     MyClassListener.names.clear();
     TestNG tng = create(ClassListenerSample.class);
     TestListenerAdapter adapter = new TestListenerAdapter();
-    tng.addListener(adapter);
+    tng.addListener((ITestNGListener) adapter);
     tng.run();
     assertThat(adapter.getFailedTests()).isEmpty();
     assertThat(adapter.getSkippedTests()).isEmpty();
@@ -193,7 +194,7 @@ public class ListenerTest extends SimpleBaseTest {
   public void methodListenersShouldNotBeCalledForSkippedMethods() {
     GitHub911Listener listener = new GitHub911Listener();
     TestNG tng = create(GitHub911Sample.class);
-    tng.addListener(listener);
+    tng.addListener((ITestNGListener) listener);
     tng.run();
     Assert.assertEquals(listener.onStart, 1);
     Assert.assertEquals(listener.onFinish, 1);
@@ -208,11 +209,44 @@ public class ListenerTest extends SimpleBaseTest {
   public void setStatusShouldWorkInListener() {
     SetStatusListener listener = new SetStatusListener();
     TestNG tng = create(SetStatusSample.class);
-    tng.addListener(listener);
+    tng.addListener((ITestNGListener) listener);
     tng.run();
     Assert.assertEquals(listener.getContext().getFailedTests().size(), 0);
     Assert.assertEquals(listener.getContext().getFailedButWithinSuccessPercentageTests().size(), 0);
     Assert.assertEquals(listener.getContext().getSkippedTests().size(), 0);
     Assert.assertEquals(listener.getContext().getPassedTests().size(), 1);
   }
+
+  @Test(description = "GITHUB-1084: Using deprecated addListener methods should not register many times")
+  public void listenerRegistration() {
+    MultiListener listener = new MultiListener();
+    TestNG tng = create(SimpleSample.class);
+    // Keep using deprecated addListener methods. It is what the test is testing
+    tng.addListener((ISuiteListener) listener);
+    tng.addListener((ITestListener) listener);
+    tng.addInvokedMethodListener(listener);
+    tng.run();
+    Assert.assertEquals(listener.getOnSuiteStartCount(), 1);
+    Assert.assertEquals(listener.getOnSuiteFinishCount(), 1);
+    Assert.assertEquals(listener.getOnTestStartCount(), 1);
+    Assert.assertEquals(listener.getOnTestFinishCount(), 1);
+    Assert.assertEquals(listener.getBeforeInvocationCount(), 1);
+    Assert.assertEquals(listener.getAfterInvocationCount(), 1);
+    Assert.assertEquals(listener.getOnMethodTestStartCount(), 1);
+    Assert.assertEquals(listener.getOnMethodTestSuccessCount(), 1);
+  }
+
+  @Test
+  public void testListenerCallInvocation() {
+    XmlSuite suite = createXmlSuite("suite956", "test956", TestClassContainer.FirstTestClass.class,
+        TestClassContainer.SecondTestClass.class);
+    TestNG tng = create(suite);
+    ListenerFor956 listener = new ListenerFor956();
+    tng.addListener((ITestNGListener) listener);
+    tng.run();
+    List<String> messages = listener.getMessages();
+    Assert.assertEquals(messages.size(), 1);
+    Assert.assertEquals(messages.get(0), "Executing test956");
+  }
+
 }
